package net.wanji.opt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.dto.CrossIdAndDirDTO;
import net.wanji.opt.dto.CrossIdAndSchemeIdDTO;
import net.wanji.opt.dto.SaveLaneInfoDTO;
import net.wanji.opt.dto.SendManualDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.OptEffectVO;
import net.wanji.opt.vo.SchemeOptVO;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 诊断优化
 *
 * @author Kent HAN
 */
@Api(value = "DiagnoController", description = "诊断优化")
@RequestMapping("/diagno")
@RestController
public class DiagnoController {

    private final DiagnoService diagnoService;

    public DiagnoController(DiagnoService diagnoService) {
        this.diagnoService = diagnoService;
    }

    @ApiOperation(value = "查询路口当前场景、优化策略、优化方法", notes = "查询路口当前场景、优化策略、优化方法",
            response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryCrossScene",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject queryCrossScene(@RequestBody CrossIdDTO crossIdDTO) {
        AddOrUpdateSceneDTO addOrUpdateSceneDTO = diagnoService.queryCrossScene(crossIdDTO);
        return JsonViewObject.newInstance().success(addOrUpdateSceneDTO);
    }

    @ApiOperation(value = "查询当前路口相邻路口", notes = "查询当前路口相邻路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryNeighborCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossIdAndLocationVO.class),
    })
    public JsonViewObject queryNeighborCross(@RequestBody CrossIdDTO crossIdDTO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = diagnoService.queryNeighborCross(crossIdDTO);
        return JsonViewObject.newInstance().success(crossIdAndLocationVOList);
    }

    @ApiOperation(value = "查询方案优化", notes = "查询方案优化", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/querySchemeOpt",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeOptVO.class),
    })
    public JsonViewObject querySchemeOpt(@RequestBody @Valid CrossIdDTO crossIdDTO) {
        SchemeOptVO schemeOptVO = diagnoService.querySchemeOpt(crossIdDTO);
        return JsonViewObject.newInstance().success(schemeOptVO);
    }

    @ApiOperation(value = "根据强制方案返回调整时间", notes = "根据强制方案返回调整时间", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryOptTime",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeOptVO.PhaseOptTime.class),
    })
    public JsonViewObject queryOptTime(@RequestBody CrossIdAndSchemeIdDTO crossIdAndSchemeIdDTO) {
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = diagnoService.queryOptTime(crossIdAndSchemeIdDTO);
        return JsonViewObject.newInstance().success(phaseOptTimeList);
    }

    @ApiOperation(value = "下发手动干预", notes = "下发手动干预", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/sendManual",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject sendManual(@RequestBody SendManualDTO sendManualDTO) throws Exception {
        diagnoService.sendManual(sendManualDTO);
        return JsonViewObject.newInstance().success();
    }

    @ApiOperation(value = "恢复下发", notes = "恢复下发", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/restoreSend",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject restoreSend(@RequestBody CrossIdDTO crossIdDTO) throws Exception {
        diagnoService.restoreSend(crossIdDTO);
        return JsonViewObject.newInstance().success();
    }

    @ApiOperation(value = "优化效果", notes = "优化效果", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/optEffect",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptEffectVO.class),
    })
    public JsonViewObject optEffect(@RequestBody CrossIdAndDirDTO crossIdAndDirDTO) {
        OptEffectVO optEffectVO = diagnoService.optEffect(crossIdAndDirDTO);
        return JsonViewObject.newInstance().success(optEffectVO);
    }

    @AspectLog(description = "路口信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "路口信息", notes = "路口信息", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/listLaneInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SaveLaneInfoDTO.class),
    })
    public JsonViewObject listLaneInfo(@RequestBody @Valid CrossIdDTO crossIdDTO) {
        SaveLaneInfoDTO saveLaneInfoDTO = diagnoService.listLaneInfo(crossIdDTO);
        return JsonViewObject.newInstance().success(saveLaneInfoDTO);
    }
}